%global redis_modules_abi 1
%global redis_modules_dir %{_libdir}/%{name}/modules
%global make_flags    DEBUG="" V="echo" LDFLAGS="%{?__global_ldflags}" CFLAGS+="%{optflags} -fPIC" INSTALL="install -p" PREFIX=%{buildroot}%{_prefix} BUILD_WITH_SYSTEMD=yes BUILD_TLS=yes

Summary:       A persistent key-value database
Name:          redis
Version:       7.0.15
Release:       3%{?dist}
License:       BSD and MIT
URL:           https://redis.io
Source0:       https://download.redis.io/releases/%{name}-%{version}.tar.gz
Source1:       %{name}.logrotate
Source2:       %{name}-sentinel.service
Source3:       %{name}.service
Source6:       %{name}-shutdown
Source7:       %{name}-limit-systemd
Source9:       macros.%{name}

Patch0001:         0001-1st-man-pageis-for-redis-cli-redis-benchmark-redis-c.patch
Patch0002:         0002-deps-jemalloc-Do-not-force-building-in-gnu99-mode.patch
Patch0003:         0001-redis-7.0.15-add-support-for-loongarch64.patch

BuildRequires: make gcc systemd-devel systemd-rpm-macros openssl-devel pkgconfig(libsystemd) procps-ng tcl

Requires:      /bin/awk logrotate
Requires(pre): shadow-utils

%{?systemd_requires}

Provides:      bundled(hiredis) = 0.14.0
Provides:      bundled(jemalloc) = 5.2.1
Provides:      bundled(lua-libs) = 5.1.5
Provides:      bundled(linenoise) = 1.0
Provides:      bundled(lzf)
Provides:      bundled(hdr_histogram) = 0.11.0
Provides:      redis(modules_abi) = %{redis_modules_abi}
Obsoletes:     %{name}-doc < 7.0.12-1

%description
Redis is an advanced key-value store. It is often referred to as a data structure server since keys can
contain strings, hashes, lists, sets and sorted sets.

You can run atomic operations on these types, like appending to a string; incrementing the value in a hash;
pushing to a list; computing set intersection, union and difference; or getting the member with highest 
ranking in a sorted set.

In order to achieve its outstanding performance, Redis works with an in-memory dataset. Depending on your use case,
you can persist it either by dumping the dataset to disk every once in a while, or by appending each command to a log.

Redis also supports trivial-to-setup master-slave replication, with very fast non-blocking first synchronization,
auto-reconnection on net split and so forth.

Other features include Transactions, Pub/Sub, Lua scripting, Keys with a limited time-to-live, and configuration settings
to make Redis behave like a cache.

You can use Redis from most programming languages also.

%package           devel
Summary:           Development header for Redis module development
Provides:          %{name}-static = %{version}-%{release}

%description       devel
Header file required for building loadable Redis modules.

%prep
%autosetup -n %{name}-%{version} -p1

mv deps/lua/COPYRIGHT    COPYRIGHT-lua
mv deps/jemalloc/COPYING COPYING-jemalloc
mv deps/hiredis/COPYING  COPYING-hiredis
mv deps/hdr_histogram/LICENSE.txt LICENSE-hdrhistogram
mv deps/hdr_histogram/COPYING.txt COPYING-hdrhistogram

sed -i -e 's|^logfile .*$|logfile /var/log/redis/redis.log|g' redis.conf
sed -i -e 's|^logfile .*$|logfile /var/log/redis/sentinel.log|g' sentinel.conf
sed -i -e 's|^dir .*$|dir /var/lib/redis|g' redis.conf

%ifarch x86_64
sed -e 's/--with-lg-quantum/--with-lg-page=12 --with-lg-quantum/' -i deps/Makefile
%endif
%ifarch ppc64le aarch64
sed -e 's/--with-lg-quantum/--with-lg-page=16 --with-lg-quantum/' -i deps/Makefile
%endif

api=`sed -n -e 's/#define REDISMODULE_APIVER_[0-9][0-9]* //p' src/redismodule.h`
if test "$api" != "%{redis_modules_abi}"; then
   exit 1
fi

%build
%make_build %{make_flags} all

%install
%make_install %{make_flags}

install -d %{buildroot}%{_sharedstatedir}/%{name}
install -d %{buildroot}%{_localstatedir}/log/%{name}
install -d %{buildroot}%{_localstatedir}/run/%{name}
install -d %{buildroot}%{redis_modules_dir}

install -pDm644 %{S:1} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}
install -pDm640 %{name}.conf  %{buildroot}%{_sysconfdir}/%{name}/%{name}.conf
install -pDm640 sentinel.conf %{buildroot}%{_sysconfdir}/%{name}/sentinel.conf

install -d %{buildroot}%{_unitdir}
install -pm644 %{S:3} %{buildroot}%{_unitdir}
install -pm644 %{S:2} %{buildroot}%{_unitdir}

install -p -D -m 644 %{S:7} %{buildroot}%{_sysconfdir}/systemd/system/%{name}.service.d/limit.conf
install -p -D -m 644 %{S:7} %{buildroot}%{_sysconfdir}/systemd/system/%{name}-sentinel.service.d/limit.conf

chmod 755 %{buildroot}%{_bindir}/%{name}-*

install -pDm755 %{S:6} %{buildroot}%{_libexecdir}/%{name}-shutdown
install -pDm644 src/%{name}module.h %{buildroot}%{_includedir}/%{name}module.h

man=$(dirname %{buildroot}%{_mandir})
for page in man/man?/*; do
    install -Dpm644 $page $man/$page
done
ln -s redis-server.1 %{buildroot}%{_mandir}/man1/redis-sentinel.1
ln -s redis.conf.5   %{buildroot}%{_mandir}/man5/redis-sentinel.conf.5

for page in 00-RELEASENOTES BUGS MANIFESTO *.md; do
    install -Dpm644 $page %{buildroot}/%{_docdir}/%{name}/$page
done

for page in $(find doc -name \*.md | sed -e 's|.md$||g'); do
    base=$(echo $page | sed -e 's|doc/||g')
    install -Dpm644 $page.md %{buildroot}/%{_docdir}/%{name}/$base.md
done

install -d %{buildroot}%{_rpmconfigdir}/macros.d
install -pDm644 %{S:9} %{buildroot}%{_rpmconfigdir}/macros.d/macros.%{name}

%ifnarch loongarch64
%check
taskset -c 1 make %{make_flags} test
make %{make_flags} test-sentinel
%endif

%pre
getent group %{name} &> /dev/null || \
groupadd -r %{name} &> /dev/null
getent passwd %{name} &> /dev/null || \
useradd -r -g %{name} -d %{_sharedstatedir}/%{name} -s /sbin/nologin \
-c 'Redis Database Server' %{name} &> /dev/null
exit 0

%post
if [ -f %{_sysconfdir}/%{name}.conf -a ! -L %{_sysconfdir}/%{name}.conf ]; then
  if [ -f %{_sysconfdir}/%{name}/%{name}.conf.rpmnew ]; then
    rm    %{_sysconfdir}/%{name}/%{name}.conf.rpmnew
  fi
  if [ -f %{_sysconfdir}/%{name}/%{name}.conf ]; then
    mv    %{_sysconfdir}/%{name}/%{name}.conf %{_sysconfdir}/%{name}/%{name}.conf.rpmnew
  fi
  mv %{_sysconfdir}/%{name}.conf %{_sysconfdir}/%{name}/%{name}.conf
  echo -e "\nWarning: %{name} configuration is now in %{_sysconfdir}/%{name} directory\n"
fi
if [ -f %{_sysconfdir}/%{name}-sentinel.conf  -a ! -L %{_sysconfdir}/%{name}-sentinel.conf  ]; then
  if [ -f %{_sysconfdir}/%{name}/sentinel.conf.rpmnew ]; then
    rm    %{_sysconfdir}/%{name}/sentinel.conf.rpmnew
  fi
  if [ -f %{_sysconfdir}/%{name}/sentinel.conf ]; then
    mv    %{_sysconfdir}/%{name}/sentinel.conf %{_sysconfdir}/%{name}/sentinel.conf.rpmnew
  fi
  mv %{_sysconfdir}/%{name}-sentinel.conf %{_sysconfdir}/%{name}/sentinel.conf
fi
%systemd_post %{name}.service
%systemd_post %{name}-sentinel.service

%preun
%systemd_preun %{name}.service
%systemd_preun %{name}-sentinel.service

%postun
%systemd_postun_with_restart %{name}.service
%systemd_postun_with_restart %{name}-sentinel.service

%files
%license COPYING COPYRIGHT-lua COPYING-jemalloc COPYING-hiredis LICENSE-hdrhistogram COPYING-hdrhistogram
%config(noreplace) %{_sysconfdir}/logrotate.d/%{name}
%attr(0750, redis, root) %dir %{_sysconfdir}/%{name}
%attr(0640, redis, root) %config(noreplace) %{_sysconfdir}/%{name}/%{name}.conf
%attr(0640, redis, root) %config(noreplace) %{_sysconfdir}/%{name}/sentinel.conf
%dir %attr(0750, redis, redis) %{_libdir}/%{name}
%dir %attr(0750, redis, redis) %{redis_modules_dir}
%dir %attr(0750, redis, redis) %{_sharedstatedir}/%{name}
%dir %attr(0750, redis, redis) %{_localstatedir}/log/%{name}
%dir %{_sysconfdir}/systemd/system/%{name}.service.d
%config(noreplace) %{_sysconfdir}/systemd/system/%{name}.service.d/limit.conf
%dir %{_sysconfdir}/systemd/system/%{name}-sentinel.service.d
%config(noreplace) %{_sysconfdir}/systemd/system/%{name}-sentinel.service.d/limit.conf
%{_bindir}/%{name}-*
%{_libexecdir}/%{name}-*
%{_unitdir}/%{name}.service
%{_unitdir}/%{name}-sentinel.service
%dir %attr(0755, redis, redis) %ghost %{_localstatedir}/run/%{name}
%{_mandir}/man1/%{name}*
%{_mandir}/man5/%{name}*

%files devel
%license COPYING
%docdir %{_docdir}/%{name}
%{_docdir}/%{name}
%{_includedir}/%{name}module.h
%{_rpmconfigdir}/macros.d/*


%changelog
* Wed Jul 03 2024 zhailiangliang <zhailiangliang@loongson.cn> - 7.0.15-3
- [Type] other
- [DESC] disable test for loongarch64

* Mon Mar 18 2024 doupengda <doupengda@loongson.cn> - 7.0.15-2
- add support for loongarch64

* Thu Jan 18 2024 jackeyji <jackeyji@tencent.com> - 7.0.15-1
- Upgrade to upstream version 7.0.15
- Fix CVE-2023-41056

* Wed Nov 08 2023 Xiaojie Chen <jackxjchen@tencent.com> - 7.0.14-1
- Upgrade to upstream version 7.0.14
- Fix CVE-2023-41053 and CVE-2023-45145
- Set jemalloc page size

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 7.0.12-2
- Rebuilt for OpenCloudOS Stream 23.09

* Mon Sep 4 2023 Shuo Wang <abushwang@tencent.com> - 7.0.12-1
- update to 7.0.12

* Thu Aug 03 2023 cunshunxia <cunshunxia@tencent.com> - 7.0.5-4
- Rebuilt for tcl 8.6.13

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 7.0.5-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 7.0.5-2
- Rebuilt for OpenCloudOS Stream 23

* Fri Dec 9 2022 Shuo Wang <abushwang@tencent.com> - 7.0.5-1
- initial build
